# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
from .legacy_guideline import LegacyGuideline
import typing
from .legacy_guideline_connection import LegacyGuidelineConnection
from .guideline_tool_association import GuidelineToolAssociation
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class LegacyGuidelineWithConnectionsAndToolAssociations(UniversalBaseModel):
    """
    A Guideline with its connections and tool associations.
    """

    guideline: LegacyGuideline
    connections: typing.List[LegacyGuidelineConnection]
    tool_associations: typing.List[GuidelineToolAssociation]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

SessionStatusDto = typing.Union[
    typing.Literal[
        "acknowledged", "cancelled", "processing", "ready", "typing", "error"
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
from .legacy_guideline_invoice_data import LegacyGuidelineInvoiceData
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class LegacyInvoiceData(UniversalBaseModel):
    """
    Contains the relevant invoice data.

    At this point only `guideline` is supported.
    """

    guideline: typing.Optional[LegacyGuidelineInvoiceData] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import pydantic
from .tool_id import ToolId
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class GuidelineToolAssociation(UniversalBaseModel):
    """
    Represents an association between a Guideline and a Tool, enabling automatic tool invocation
    when the Guideline's conditions are met.
    """

    id: str = pydantic.Field()
    """
    Unique identifier for the association between a tool and a guideline
    """

    guideline_id: str = pydantic.Field()
    """
    Unique identifier for the guideline
    """

    tool_id: ToolId

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing
from .tool_id import ToolId
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class LegacyContextVariableCreationParams(UniversalBaseModel):
    """
    Parameters for creating a new context variable.
    """

    name: str = pydantic.Field()
    """
    Name of the context variable
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Description of the context variable's purpose
    """

    tool_id: typing.Optional[ToolId] = None
    freshness_rules: typing.Optional[str] = pydantic.Field(default=None)
    """
    Cron expression defining the freshness rules
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import pydantic
import datetime as dt
import typing
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Customer(UniversalBaseModel):
    """
    Represents a customer in the system.

    Customers are entities that interact with agents through sessions. Each customer
    can have metadata stored in the metadata field and can be tagged for categorization.
    """

    id: str = pydantic.Field()
    """
    Unique identifier for the customer
    """

    creation_utc: dt.datetime = pydantic.Field()
    """
    UTC timestamp of when the customer was created
    """

    name: str = pydantic.Field()
    """
    An arbitrary string that identifies and/or describes the customer
    """

    metadata: typing.Dict[str, str] = pydantic.Field()
    """
    Key-value pairs (`str: str`) to describe the customer
    """

    tags: typing.List[str] = pydantic.Field()
    """
    Collection of ids of tags that describe the customer
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.customer import Customer
from ..core.pydantic_utilities import parse_obj_as
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..core.jsonable_encoder import jsonable_encoder
from ..errors.not_found_error import NotFoundError
from ..types.customer_metadata_update_params import CustomerMetadataUpdateParams
from ..types.customer_tag_update_params import CustomerTagUpdateParams
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CustomersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[Customer]:
        """
        Retrieves a list of all customers in the system.

        Returns an empty list if no customers exist.
        Customers are returned in no guaranteed order.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Customer]
            List of all customers in the system.

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.customers.list()
        """
        _response = self._client_wrapper.httpx_client.request(
            "customers",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[Customer],
                    parse_obj_as(
                        type_=typing.List[Customer],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        name: str,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[str]]] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Customer:
        """
        Creates a new customer in the system.

        A customer may be created with as little as a `name`.
        `metadata` key-value pairs and additional `tags` may be attached to a customer.

        Parameters
        ----------
        name : str
            An arbitrary string that identifies and/or describes the customer

        metadata : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Key-value pairs (`str: str`) to describe the customer

        tags : typing.Optional[typing.Sequence[str]]
            Collection of ids of tags that describe the customer

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Customer
            Customer successfully created. Returns the new customer object.

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.customers.create(
            name="Scooby",
            metadata={"VIP": "Yes", "email": "scooby@dooby.do"},
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "customers",
            method="POST",
            json={
                "name": name,
                "metadata": metadata,
                "tags": tags,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Customer,
                    parse_obj_as(
                        type_=Customer,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def retrieve(
        self,
        customer_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Customer:
        """
        Retrieves details of a specific customer by ID.

        Returns a complete customer object including their metadata and tags.
        The customer must exist in the system.

        Parameters
        ----------
        customer_id : str
            Unique identifier for the customer

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Customer
            Customer details successfully retrieved. Returns the Customer object.

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.customers.retrieve(
            customer_id="ck_IdAXUtp",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"customers/{jsonable_encoder(customer_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Customer,
                    parse_obj_as(
                        type_=Customer,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self,
        customer_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Deletes a customer from the agent.

        Deleting a non-existent customer will return 404.
        No content will be returned from a successful deletion.

        Parameters
        ----------
        customer_id : str
            Unique identifier for the customer

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.customers.delete(
            customer_id="ck_IdAXUtp",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"customers/{jsonable_encoder(customer_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        customer_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        metadata: typing.Optional[CustomerMetadataUpdateParams] = OMIT,
        tags: typing.Optional[CustomerTagUpdateParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Customer:
        """
        Updates an existing customer's attributes.

        Only provided attributes will be updated; others remain unchanged.
        The customer's ID and creation timestamp cannot be modified.
        Extra metadata and tags can be added or removed independently.

        Parameters
        ----------
        customer_id : str
            Unique identifier for the customer

        name : typing.Optional[str]
            An arbitrary string that identifies and/or describes the customer

        metadata : typing.Optional[CustomerMetadataUpdateParams]

        tags : typing.Optional[CustomerTagUpdateParams]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Customer
            Customer successfully updated. Returns the updated Customer object.

        Examples
        --------
        from parlant.client import CustomerTagUpdateParams, ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.customers.update(
            customer_id="ck_IdAXUtp",
            name="Scooby",
            tags=CustomerTagUpdateParams(
                add=["t9a8g703f4", "tag_456abc"],
                remove=["tag_789def", "tag_012ghi"],
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"customers/{jsonable_encoder(customer_id)}",
            method="PATCH",
            json={
                "name": name,
                "metadata": convert_and_respect_annotation_metadata(
                    object_=metadata,
                    annotation=CustomerMetadataUpdateParams,
                    direction="write",
                ),
                "tags": convert_and_respect_annotation_metadata(
                    object_=tags, annotation=CustomerTagUpdateParams, direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Customer,
                    parse_obj_as(
                        type_=Customer,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncCustomersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[Customer]:
        """
        Retrieves a list of all customers in the system.

        Returns an empty list if no customers exist.
        Customers are returned in no guaranteed order.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Customer]
            List of all customers in the system.

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.customers.list()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "customers",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[Customer],
                    parse_obj_as(
                        type_=typing.List[Customer],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        name: str,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[str]]] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Customer:
        """
        Creates a new customer in the system.

        A customer may be created with as little as a `name`.
        `metadata` key-value pairs and additional `tags` may be attached to a customer.

        Parameters
        ----------
        name : str
            An arbitrary string that identifies and/or describes the customer

        metadata : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Key-value pairs (`str: str`) to describe the customer

        tags : typing.Optional[typing.Sequence[str]]
            Collection of ids of tags that describe the customer

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Customer
            Customer successfully created. Returns the new customer object.

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.customers.create(
                name="Scooby",
                metadata={"VIP": "Yes", "email": "scooby@dooby.do"},
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "customers",
            method="POST",
            json={
                "name": name,
                "metadata": metadata,
                "tags": tags,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Customer,
                    parse_obj_as(
                        type_=Customer,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def retrieve(
        self,
        customer_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Customer:
        """
        Retrieves details of a specific customer by ID.

        Returns a complete customer object including their metadata and tags.
        The customer must exist in the system.

        Parameters
        ----------
        customer_id : str
            Unique identifier for the customer

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Customer
            Customer details successfully retrieved. Returns the Customer object.

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.customers.retrieve(
                customer_id="ck_IdAXUtp",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"customers/{jsonable_encoder(customer_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Customer,
                    parse_obj_as(
                        type_=Customer,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self,
        customer_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Deletes a customer from the agent.

        Deleting a non-existent customer will return 404.
        No content will be returned from a successful deletion.

        Parameters
        ----------
        customer_id : str
            Unique identifier for the customer

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.customers.delete(
                customer_id="ck_IdAXUtp",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"customers/{jsonable_encoder(customer_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        customer_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        metadata: typing.Optional[CustomerMetadataUpdateParams] = OMIT,
        tags: typing.Optional[CustomerTagUpdateParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Customer:
        """
        Updates an existing customer's attributes.

        Only provided attributes will be updated; others remain unchanged.
        The customer's ID and creation timestamp cannot be modified.
        Extra metadata and tags can be added or removed independently.

        Parameters
        ----------
        customer_id : str
            Unique identifier for the customer

        name : typing.Optional[str]
            An arbitrary string that identifies and/or describes the customer

        metadata : typing.Optional[CustomerMetadataUpdateParams]

        tags : typing.Optional[CustomerTagUpdateParams]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Customer
            Customer successfully updated. Returns the updated Customer object.

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient, CustomerTagUpdateParams

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.customers.update(
                customer_id="ck_IdAXUtp",
                name="Scooby",
                tags=CustomerTagUpdateParams(
                    add=["t9a8g703f4", "tag_456abc"],
                    remove=["tag_789def", "tag_012ghi"],
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"customers/{jsonable_encoder(customer_id)}",
            method="PATCH",
            json={
                "name": name,
                "metadata": convert_and_respect_annotation_metadata(
                    object_=metadata,
                    annotation=CustomerMetadataUpdateParams,
                    direction="write",
                ),
                "tags": convert_and_respect_annotation_metadata(
                    object_=tags, annotation=CustomerTagUpdateParams, direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Customer,
                    parse_obj_as(
                        type_=Customer,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class LegacyGuideline(UniversalBaseModel):
    """
    Assigns an id to the condition-action pair
    """

    id: str = pydantic.Field()
    """
    Unique identifier for the guideline
    """

    condition: str = pydantic.Field()
    """
    If this condition is satisfied, the action will be performed
    """

    action: str = pydantic.Field()
    """
    This action will be performed if the condition is satisfied
    """

    enabled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the guideline is enabled
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .types import (
    Agent,
    AgentMessageGuideline,
    AgentMessageGuidelineRationaleDto,
    AgentTagUpdateParams,
    CannedResponse,
    CannedResponseField,
    CannedResponseTagUpdateParams,
    Capability,
    CapabilityTagUpdateParams,
    CoherenceCheck,
    CoherenceCheckKindDto,
    CompositionModeDto,
    ConnectionProposition,
    ConnectionPropositionKindDto,
    ConsumptionOffsets,
    ConsumptionOffsetsUpdateParams,
    ContextVariable,
    ContextVariableReadResult,
    ContextVariableTagsUpdateParams,
    ContextVariableValue,
    Customer,
    CustomerMetadataUpdateParams,
    CustomerTagUpdateParams,
    Evaluation,
    EvaluationStatusDto,
    Event,
    EventKindDto,
    EventSourceDto,
    Guideline,
    GuidelineContent,
    GuidelineInvoiceData,
    GuidelineMetadataUpdateParams,
    GuidelinePayload,
    GuidelinePayloadOperationDto,
    GuidelineTagsUpdateParams,
    GuidelineToolAssociation,
    GuidelineToolAssociationUpdateParams,
    GuidelineWithRelationshipsAndToolAssociations,
    HttpValidationError,
    Invoice,
    InvoiceData,
    Journey,
    JourneyConditionUpdateParams,
    JourneyTagUpdateParams,
    LegacyContextVariable,
    LegacyContextVariableCreationParams,
    LegacyContextVariableReadResult,
    LegacyContextVariableUpdateParams,
    LegacyEvaluation,
    LegacyEvaluationCreationParams,
    LegacyGuideline,
    LegacyGuidelineConnection,
    LegacyGuidelineConnectionAddition,
    LegacyGuidelineConnectionUpdateParams,
    LegacyGuidelineCreationParams,
    LegacyGuidelineCreationResult,
    LegacyGuidelineInvoiceData,
    LegacyGuidelinePayload,
    LegacyGuidelineUpdateParams,
    LegacyGuidelineWithConnectionsAndToolAssociations,
    LegacyInvoice,
    LegacyInvoiceData,
    LegacyPayload,
    LegacyTerm,
    LegacyTermCreationParams,
    LegacyTermUpdateParams,
    McpServiceParams,
    Moderation,
    OpenApiServiceParams,
    Participant,
    Payload,
    PayloadKindDto,
    Relationship,
    RelationshipKindDto,
    SdkServiceParams,
    Service,
    Session,
    SessionModeDto,
    SessionStatusDto,
    Tag,
    Term,
    TermTagsUpdateParams,
    Tool,
    ToolId,
    ToolParameter,
    ToolParameterDtoEnumItem,
    ToolParameterTypeDto,
    ToolServiceKindDto,
    ValidationError,
    ValidationErrorLocItem,
)
from .errors import (
    GatewayTimeoutError,
    NotFoundError,
    ServiceUnavailableError,
    UnprocessableEntityError,
)
from . import (
    agents,
    canned_responses,
    capabilities,
    context_variables,
    customers,
    evaluations,
    glossary,
    guidelines,
    journeys,
    relationships,
    services,
    sessions,
    tags,
)
from .client import AsyncParlantClient, ParlantClient

__all__ = [
    "Agent",
    "AgentMessageGuideline",
    "AgentMessageGuidelineRationaleDto",
    "AgentTagUpdateParams",
    "AsyncParlantClient",
    "CannedResponse",
    "CannedResponseField",
    "CannedResponseTagUpdateParams",
    "Capability",
    "CapabilityTagUpdateParams",
    "CoherenceCheck",
    "CoherenceCheckKindDto",
    "CompositionModeDto",
    "ConnectionProposition",
    "ConnectionPropositionKindDto",
    "ConsumptionOffsets",
    "ConsumptionOffsetsUpdateParams",
    "ContextVariable",
    "ContextVariableReadResult",
    "ContextVariableTagsUpdateParams",
    "ContextVariableValue",
    "Customer",
    "CustomerMetadataUpdateParams",
    "CustomerTagUpdateParams",
    "Evaluation",
    "EvaluationStatusDto",
    "Event",
    "EventKindDto",
    "EventSourceDto",
    "GatewayTimeoutError",
    "Guideline",
    "GuidelineContent",
    "GuidelineInvoiceData",
    "GuidelineMetadataUpdateParams",
    "GuidelinePayload",
    "GuidelinePayloadOperationDto",
    "GuidelineTagsUpdateParams",
    "GuidelineToolAssociation",
    "GuidelineToolAssociationUpdateParams",
    "GuidelineWithRelationshipsAndToolAssociations",
    "HttpValidationError",
    "Invoice",
    "InvoiceData",
    "Journey",
    "JourneyConditionUpdateParams",
    "JourneyTagUpdateParams",
    "LegacyContextVariable",
    "LegacyContextVariableCreationParams",
    "LegacyContextVariableReadResult",
    "LegacyContextVariableUpdateParams",
    "LegacyEvaluation",
    "LegacyEvaluationCreationParams",
    "LegacyGuideline",
    "LegacyGuidelineConnection",
    "LegacyGuidelineConnectionAddition",
    "LegacyGuidelineConnectionUpdateParams",
    "LegacyGuidelineCreationParams",
    "LegacyGuidelineCreationResult",
    "LegacyGuidelineInvoiceData",
    "LegacyGuidelinePayload",
    "LegacyGuidelineUpdateParams",
    "LegacyGuidelineWithConnectionsAndToolAssociations",
    "LegacyInvoice",
    "LegacyInvoiceData",
    "LegacyPayload",
    "LegacyTerm",
    "LegacyTermCreationParams",
    "LegacyTermUpdateParams",
    "McpServiceParams",
    "Moderation",
    "NotFoundError",
    "OpenApiServiceParams",
    "ParlantClient",
    "Participant",
    "Payload",
    "PayloadKindDto",
    "Relationship",
    "RelationshipKindDto",
    "SdkServiceParams",
    "Service",
    "ServiceUnavailableError",
    "Session",
    "SessionModeDto",
    "SessionStatusDto",
    "Tag",
    "Term",
    "TermTagsUpdateParams",
    "Tool",
    "ToolId",
    "ToolParameter",
    "ToolParameterDtoEnumItem",
    "ToolParameterTypeDto",
    "ToolServiceKindDto",
    "UnprocessableEntityError",
    "ValidationError",
    "ValidationErrorLocItem",
    "agents",
    "canned_responses",
    "capabilities",
    "context_variables",
    "customers",
    "evaluations",
    "glossary",
    "guidelines",
    "journeys",
    "relationships",
    "services",
    "sessions",
    "tags",
]



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
from .context_variable import ContextVariable
import typing
from .context_variable_value import ContextVariableValue
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ContextVariableReadResult(UniversalBaseModel):
    """
    Complete context variable data including its values.
    """

    context_variable: ContextVariable
    key_value_pairs: typing.Optional[
        typing.Dict[str, typing.Optional[ContextVariableValue]]
    ] = pydantic.Field(default=None)
    """
    Collection of key-value pairs associated with the variable
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.context_variable import ContextVariable
from ..core.pydantic_utilities import parse_obj_as
from ..errors.not_found_error import NotFoundError
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.tool_id import ToolId
from ..core.serialization import convert_and_respect_annotation_metadata
from ..types.context_variable_read_result import ContextVariableReadResult
from ..core.jsonable_encoder import jsonable_encoder
from ..types.context_variable_tags_update_params import ContextVariableTagsUpdateParams
from ..types.context_variable_value import ContextVariableValue
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ContextVariablesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        tag_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[ContextVariable]:
        """
        Lists all context variables set for the provided tag or all context variables if no tag is provided

        Parameters
        ----------
        tag_id : typing.Optional[str]
            The tag ID to filter context variables by

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[ContextVariable]
            List of all context variables

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.context_variables.list()
        """
        _response = self._client_wrapper.httpx_client.request(
            "context-variables",
            method="GET",
            params={
                "tag_id": tag_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[ContextVariable],
                    parse_obj_as(
                        type_=typing.List[ContextVariable],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        name: str,
        description: typing.Optional[str] = OMIT,
        tool_id: typing.Optional[ToolId] = OMIT,
        freshness_rules: typing.Optional[str] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ContextVariable:
        """
        Creates a new context variable

        Example uses:
        - Track subscription tiers to control feature access
        - Store usage patterns for personalized recommendations
        - Remember preferences for tailored responses

        Parameters
        ----------
        name : str
            Name of the context variable

        description : typing.Optional[str]
            Description of the context variable's purpose

        tool_id : typing.Optional[ToolId]

        freshness_rules : typing.Optional[str]
            Cron expression defining the freshness rules

        tags : typing.Optional[typing.Sequence[str]]
            List of tags associated with the context variable

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ContextVariable
            Context variable type successfully created

        Examples
        --------
        from parlant.client import ParlantClient, ToolId

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.context_variables.create(
            name="UserBalance",
            description="Stores the account balances of users",
            tool_id=ToolId(
                service_name="finance_service",
                tool_name="balance_checker",
            ),
            freshness_rules="30 2 * * *",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "context-variables",
            method="POST",
            json={
                "name": name,
                "description": description,
                "tool_id": convert_and_respect_annotation_metadata(
                    object_=tool_id, annotation=ToolId, direction="write"
                ),
                "freshness_rules": freshness_rules,
                "tags": tags,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ContextVariable,
                    parse_obj_as(
                        type_=ContextVariable,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_many(
        self,
        *,
        tag_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Deletes all context variables for the provided tag

        Parameters
        ----------
        tag_id : typing.Optional[str]
            The tag ID to filter context variables by

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.context_variables.delete_many()
        """
        _response = self._client_wrapper.httpx_client.request(
            "context-variables",
            method="DELETE",
            params={
                "tag_id": tag_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def retrieve(
        self,
        variable_id: str,
        *,
        include_values: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ContextVariableReadResult:
        """
        Retrieves a context variable's details and optionally its values.

        Can return all customer or tag values for this variable type if include_values=True.

        Parameters
        ----------
        variable_id : str
            Unique identifier for the context variable

        include_values : typing.Optional[bool]
            Whether to include variable values in the response

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ContextVariableReadResult
            Context variable details successfully retrieved

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.context_variables.retrieve(
            variable_id="v9a8r7i6b5",
            include_values=True,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"context-variables/{jsonable_encoder(variable_id)}",
            method="GET",
            params={
                "include_values": include_values,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ContextVariableReadResult,
                    parse_obj_as(
                        type_=ContextVariableReadResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self,
        variable_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Deletes a context variable

        Parameters
        ----------
        variable_id : str
            Unique identifier for the context variable

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.context_variables.delete(
            variable_id="v9a8r7i6b5",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"context-variables/{jsonable_encoder(variable_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        variable_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        tool_id: typing.Optional[ToolId] = OMIT,
        freshness_rules: typing.Optional[str] = OMIT,
        tags: typing.Optional[ContextVariableTagsUpdateParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ContextVariable:
        """
        Updates an existing context variable.

        Only provided fields will be updated; others remain unchanged.

        Parameters
        ----------
        variable_id : str
            Unique identifier for the context variable

        name : typing.Optional[str]
            Name of the context variable

        description : typing.Optional[str]
            Description of the context variable's purpose

        tool_id : typing.Optional[ToolId]

        freshness_rules : typing.Optional[str]
            Cron expression defining the freshness rules

        tags : typing.Optional[ContextVariableTagsUpdateParams]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ContextVariable
            Context variable type successfully updated

        Examples
        --------
        from parlant.client import ContextVariableTagsUpdateParams, ParlantClient, ToolId

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.context_variables.update(
            variable_id="v9a8r7i6b5",
            name="UserBalance",
            description="Stores the account balances of users",
            tool_id=ToolId(
                service_name="finance_service",
                tool_name="balance_checker",
            ),
            freshness_rules="0 8,20 * * *",
            tags=ContextVariableTagsUpdateParams(
                add=["tag:123", "tag:456"],
                remove=["tag:789", "tag:012"],
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"context-variables/{jsonable_encoder(variable_id)}",
            method="PATCH",
            json={
                "name": name,
                "description": description,
                "tool_id": convert_and_respect_annotation_metadata(
                    object_=tool_id, annotation=ToolId, direction="write"
                ),
                "freshness_rules": freshness_rules,
                "tags": convert_and_respect_annotation_metadata(
                    object_=tags,
                    annotation=ContextVariableTagsUpdateParams,
                    direction="write",
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ContextVariable,
                    parse_obj_as(
                        type_=ContextVariable,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_value(
        self,
        variable_id: str,
        key: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ContextVariableValue:
        """
        Retrieves a customer or tag value for the provided context variable

        Parameters
        ----------
        variable_id : str
            Unique identifier for the context variable

        key : str
            Key for the variable value

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ContextVariableValue
            Retrieved context value for the customer or tag

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.context_variables.get_value(
            variable_id="v9a8r7i6b5",
            key="user_1",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"context-variables/{jsonable_encoder(variable_id)}/{jsonable_encoder(key)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ContextVariableValue,
                    parse_obj_as(
                        type_=ContextVariableValue,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def set_value(
        self,
        variable_id: str,
        key: str,
        *,
        data: typing.Optional[typing.Any] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ContextVariableValue:
        """
        Updates a customer or tag value for the provided context variable

        Parameters
        ----------
        variable_id : str
            Unique identifier for the context variable

        key : str
            Key for the variable value

        data : typing.Optional[typing.Any]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ContextVariableValue
            Context value successfully updated for the customer or tag

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.context_variables.set_value(
            variable_id="v9a8r7i6b5",
            key="user_1",
            data={
                "balance": 5000.5,
                "currency": "USD",
                "last_transaction": "2024-03-23T15:30:00Z",
                "status": "active",
            },
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"context-variables/{jsonable_encoder(variable_id)}/{jsonable_encoder(key)}",
            method="PUT",
            json={
                "data": data,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ContextVariableValue,
                    parse_obj_as(
                        type_=ContextVariableValue,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_value(
        self,
        variable_id: str,
        key: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Deletes a customer or tag value for the provided context variable

        Parameters
        ----------
        variable_id : str
            Unique identifier for the context variable

        key : str
            Key for the variable value

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.context_variables.delete_value(
            variable_id="v9a8r7i6b5",
            key="user_1",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"context-variables/{jsonable_encoder(variable_id)}/{jsonable_encoder(key)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncContextVariablesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        tag_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[ContextVariable]:
        """
        Lists all context variables set for the provided tag or all context variables if no tag is provided

        Parameters
        ----------
        tag_id : typing.Optional[str]
            The tag ID to filter context variables by

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[ContextVariable]
            List of all context variables

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.context_variables.list()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "context-variables",
            method="GET",
            params={
                "tag_id": tag_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[ContextVariable],
                    parse_obj_as(
                        type_=typing.List[ContextVariable],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        name: str,
        description: typing.Optional[str] = OMIT,
        tool_id: typing.Optional[ToolId] = OMIT,
        freshness_rules: typing.Optional[str] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ContextVariable:
        """
        Creates a new context variable

        Example uses:
        - Track subscription tiers to control feature access
        - Store usage patterns for personalized recommendations
        - Remember preferences for tailored responses

        Parameters
        ----------
        name : str
            Name of the context variable

        description : typing.Optional[str]
            Description of the context variable's purpose

        tool_id : typing.Optional[ToolId]

        freshness_rules : typing.Optional[str]
            Cron expression defining the freshness rules

        tags : typing.Optional[typing.Sequence[str]]
            List of tags associated with the context variable

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ContextVariable
            Context variable type successfully created

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient, ToolId

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.context_variables.create(
                name="UserBalance",
                description="Stores the account balances of users",
                tool_id=ToolId(
                    service_name="finance_service",
                    tool_name="balance_checker",
                ),
                freshness_rules="30 2 * * *",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "context-variables",
            method="POST",
            json={
                "name": name,
                "description": description,
                "tool_id": convert_and_respect_annotation_metadata(
                    object_=tool_id, annotation=ToolId, direction="write"
                ),
                "freshness_rules": freshness_rules,
                "tags": tags,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ContextVariable,
                    parse_obj_as(
                        type_=ContextVariable,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_many(
        self,
        *,
        tag_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Deletes all context variables for the provided tag

        Parameters
        ----------
        tag_id : typing.Optional[str]
            The tag ID to filter context variables by

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.context_variables.delete_many()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "context-variables",
            method="DELETE",
            params={
                "tag_id": tag_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def retrieve(
        self,
        variable_id: str,
        *,
        include_values: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ContextVariableReadResult:
        """
        Retrieves a context variable's details and optionally its values.

        Can return all customer or tag values for this variable type if include_values=True.

        Parameters
        ----------
        variable_id : str
            Unique identifier for the context variable

        include_values : typing.Optional[bool]
            Whether to include variable values in the response

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ContextVariableReadResult
            Context variable details successfully retrieved

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.context_variables.retrieve(
                variable_id="v9a8r7i6b5",
                include_values=True,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"context-variables/{jsonable_encoder(variable_id)}",
            method="GET",
            params={
                "include_values": include_values,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ContextVariableReadResult,
                    parse_obj_as(
                        type_=ContextVariableReadResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self,
        variable_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Deletes a context variable

        Parameters
        ----------
        variable_id : str
            Unique identifier for the context variable

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.context_variables.delete(
                variable_id="v9a8r7i6b5",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"context-variables/{jsonable_encoder(variable_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        variable_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        tool_id: typing.Optional[ToolId] = OMIT,
        freshness_rules: typing.Optional[str] = OMIT,
        tags: typing.Optional[ContextVariableTagsUpdateParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ContextVariable:
        """
        Updates an existing context variable.

        Only provided fields will be updated; others remain unchanged.

        Parameters
        ----------
        variable_id : str
            Unique identifier for the context variable

        name : typing.Optional[str]
            Name of the context variable

        description : typing.Optional[str]
            Description of the context variable's purpose

        tool_id : typing.Optional[ToolId]

        freshness_rules : typing.Optional[str]
            Cron expression defining the freshness rules

        tags : typing.Optional[ContextVariableTagsUpdateParams]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ContextVariable
            Context variable type successfully updated

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient, ContextVariableTagsUpdateParams, ToolId

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.context_variables.update(
                variable_id="v9a8r7i6b5",
                name="UserBalance",
                description="Stores the account balances of users",
                tool_id=ToolId(
                    service_name="finance_service",
                    tool_name="balance_checker",
                ),
                freshness_rules="0 8,20 * * *",
                tags=ContextVariableTagsUpdateParams(
                    add=["tag:123", "tag:456"],
                    remove=["tag:789", "tag:012"],
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"context-variables/{jsonable_encoder(variable_id)}",
            method="PATCH",
            json={
                "name": name,
                "description": description,
                "tool_id": convert_and_respect_annotation_metadata(
                    object_=tool_id, annotation=ToolId, direction="write"
                ),
                "freshness_rules": freshness_rules,
                "tags": convert_and_respect_annotation_metadata(
                    object_=tags,
                    annotation=ContextVariableTagsUpdateParams,
                    direction="write",
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ContextVariable,
                    parse_obj_as(
                        type_=ContextVariable,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_value(
        self,
        variable_id: str,
        key: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ContextVariableValue:
        """
        Retrieves a customer or tag value for the provided context variable

        Parameters
        ----------
        variable_id : str
            Unique identifier for the context variable

        key : str
            Key for the variable value

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ContextVariableValue
            Retrieved context value for the customer or tag

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.context_variables.get_value(
                variable_id="v9a8r7i6b5",
                key="user_1",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"context-variables/{jsonable_encoder(variable_id)}/{jsonable_encoder(key)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ContextVariableValue,
                    parse_obj_as(
                        type_=ContextVariableValue,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def set_value(
        self,
        variable_id: str,
        key: str,
        *,
        data: typing.Optional[typing.Any] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ContextVariableValue:
        """
        Updates a customer or tag value for the provided context variable

        Parameters
        ----------
        variable_id : str
            Unique identifier for the context variable

        key : str
            Key for the variable value

        data : typing.Optional[typing.Any]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ContextVariableValue
            Context value successfully updated for the customer or tag

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.context_variables.set_value(
                variable_id="v9a8r7i6b5",
                key="user_1",
                data={
                    "balance": 5000.5,
                    "currency": "USD",
                    "last_transaction": "2024-03-23T15:30:00Z",
                    "status": "active",
                },
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"context-variables/{jsonable_encoder(variable_id)}/{jsonable_encoder(key)}",
            method="PUT",
            json={
                "data": data,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ContextVariableValue,
                    parse_obj_as(
                        type_=ContextVariableValue,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_value(
        self,
        variable_id: str,
        key: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Deletes a customer or tag value for the provided context variable

        Parameters
        ----------
        variable_id : str
            Unique identifier for the context variable

        key : str
            Key for the variable value

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.context_variables.delete_value(
                variable_id="v9a8r7i6b5",
                key="user_1",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"context-variables/{jsonable_encoder(variable_id)}/{jsonable_encoder(key)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class GuidelineInvoiceData(UniversalBaseModel):
    """
    Evaluation results for a Guideline, including action propositions
    """

    action_proposition: typing.Optional[str] = pydantic.Field(default=None)
    """
    Proposed action proposition
    """

    properties_proposition: typing.Optional[
        typing.Dict[str, typing.Optional[typing.Any]]
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .agent import Agent
from .agent_message_guideline import AgentMessageGuideline
from .agent_message_guideline_rationale_dto import AgentMessageGuidelineRationaleDto
from .agent_tag_update_params import AgentTagUpdateParams
from .canned_response import CannedResponse
from .canned_response_field import CannedResponseField
from .canned_response_tag_update_params import CannedResponseTagUpdateParams
from .capability import Capability
from .capability_tag_update_params import CapabilityTagUpdateParams
from .coherence_check import CoherenceCheck
from .coherence_check_kind_dto import CoherenceCheckKindDto
from .composition_mode_dto import CompositionModeDto
from .connection_proposition import ConnectionProposition
from .connection_proposition_kind_dto import ConnectionPropositionKindDto
from .consumption_offsets import ConsumptionOffsets
from .consumption_offsets_update_params import ConsumptionOffsetsUpdateParams
from .context_variable import ContextVariable
from .context_variable_read_result import ContextVariableReadResult
from .context_variable_tags_update_params import ContextVariableTagsUpdateParams
from .context_variable_value import ContextVariableValue
from .customer import Customer
from .customer_metadata_update_params import CustomerMetadataUpdateParams
from .customer_tag_update_params import CustomerTagUpdateParams
from .evaluation import Evaluation
from .evaluation_status_dto import EvaluationStatusDto
from .event import Event
from .event_kind_dto import EventKindDto
from .event_source_dto import EventSourceDto
from .guideline import Guideline
from .guideline_content import GuidelineContent
from .guideline_invoice_data import GuidelineInvoiceData
from .guideline_metadata_update_params import GuidelineMetadataUpdateParams
from .guideline_payload import GuidelinePayload
from .guideline_payload_operation_dto import GuidelinePayloadOperationDto
from .guideline_tags_update_params import GuidelineTagsUpdateParams
from .guideline_tool_association import GuidelineToolAssociation
from .guideline_tool_association_update_params import (
    GuidelineToolAssociationUpdateParams,
)
from .guideline_with_relationships_and_tool_associations import (
    GuidelineWithRelationshipsAndToolAssociations,
)
from .http_validation_error import HttpValidationError
from .invoice import Invoice
from .invoice_data import InvoiceData
from .journey import Journey
from .journey_condition_update_params import JourneyConditionUpdateParams
from .journey_tag_update_params import JourneyTagUpdateParams
from .legacy_context_variable import LegacyContextVariable
from .legacy_context_variable_creation_params import LegacyContextVariableCreationParams
from .legacy_context_variable_read_result import LegacyContextVariableReadResult
from .legacy_context_variable_update_params import LegacyContextVariableUpdateParams
from .legacy_evaluation import LegacyEvaluation
from .legacy_evaluation_creation_params import LegacyEvaluationCreationParams
from .legacy_guideline import LegacyGuideline
from .legacy_guideline_connection import LegacyGuidelineConnection
from .legacy_guideline_connection_addition import LegacyGuidelineConnectionAddition
from .legacy_guideline_connection_update_params import (
    LegacyGuidelineConnectionUpdateParams,
)
from .legacy_guideline_creation_params import LegacyGuidelineCreationParams
from .legacy_guideline_creation_result import LegacyGuidelineCreationResult
from .legacy_guideline_invoice_data import LegacyGuidelineInvoiceData
from .legacy_guideline_payload import LegacyGuidelinePayload
from .legacy_guideline_update_params import LegacyGuidelineUpdateParams
from .legacy_guideline_with_connections_and_tool_associations import (
    LegacyGuidelineWithConnectionsAndToolAssociations,
)
from .legacy_invoice import LegacyInvoice
from .legacy_invoice_data import LegacyInvoiceData
from .legacy_payload import LegacyPayload
from .legacy_term import LegacyTerm
from .legacy_term_creation_params import LegacyTermCreationParams
from .legacy_term_update_params import LegacyTermUpdateParams
from .mcp_service_params import McpServiceParams
from .moderation import Moderation
from .open_api_service_params import OpenApiServiceParams
from .participant import Participant
from .payload import Payload
from .payload_kind_dto import PayloadKindDto
from .relationship import Relationship
from .relationship_kind_dto import RelationshipKindDto
from .sdk_service_params import SdkServiceParams
from .service import Service
from .session import Session
from .session_mode_dto import SessionModeDto
from .session_status_dto import SessionStatusDto
from .tag import Tag
from .term import Term
from .term_tags_update_params import TermTagsUpdateParams
from .tool import Tool
from .tool_id import ToolId
from .tool_parameter import ToolParameter
from .tool_parameter_dto_enum_item import ToolParameterDtoEnumItem
from .tool_parameter_type_dto import ToolParameterTypeDto
from .tool_service_kind_dto import ToolServiceKindDto
from .validation_error import ValidationError
from .validation_error_loc_item import ValidationErrorLocItem

__all__ = [
    "Agent",
    "AgentMessageGuideline",
    "AgentMessageGuidelineRationaleDto",
    "AgentTagUpdateParams",
    "CannedResponse",
    "CannedResponseField",
    "CannedResponseTagUpdateParams",
    "Capability",
    "CapabilityTagUpdateParams",
    "CoherenceCheck",
    "CoherenceCheckKindDto",
    "CompositionModeDto",
    "ConnectionProposition",
    "ConnectionPropositionKindDto",
    "ConsumptionOffsets",
    "ConsumptionOffsetsUpdateParams",
    "ContextVariable",
    "ContextVariableReadResult",
    "ContextVariableTagsUpdateParams",
    "ContextVariableValue",
    "Customer",
    "CustomerMetadataUpdateParams",
    "CustomerTagUpdateParams",
    "Evaluation",
    "EvaluationStatusDto",
    "Event",
    "EventKindDto",
    "EventSourceDto",
    "Guideline",
    "GuidelineContent",
    "GuidelineInvoiceData",
    "GuidelineMetadataUpdateParams",
    "GuidelinePayload",
    "GuidelinePayloadOperationDto",
    "GuidelineTagsUpdateParams",
    "GuidelineToolAssociation",
    "GuidelineToolAssociationUpdateParams",
    "GuidelineWithRelationshipsAndToolAssociations",
    "HttpValidationError",
    "Invoice",
    "InvoiceData",
    "Journey",
    "JourneyConditionUpdateParams",
    "JourneyTagUpdateParams",
    "LegacyContextVariable",
    "LegacyContextVariableCreationParams",
    "LegacyContextVariableReadResult",
    "LegacyContextVariableUpdateParams",
    "LegacyEvaluation",
    "LegacyEvaluationCreationParams",
    "LegacyGuideline",
    "LegacyGuidelineConnection",
    "LegacyGuidelineConnectionAddition",
    "LegacyGuidelineConnectionUpdateParams",
    "LegacyGuidelineCreationParams",
    "LegacyGuidelineCreationResult",
    "LegacyGuidelineInvoiceData",
    "LegacyGuidelinePayload",
    "LegacyGuidelineUpdateParams",
    "LegacyGuidelineWithConnectionsAndToolAssociations",
    "LegacyInvoice",
    "LegacyInvoiceData",
    "LegacyPayload",
    "LegacyTerm",
    "LegacyTermCreationParams",
    "LegacyTermUpdateParams",
    "McpServiceParams",
    "Moderation",
    "OpenApiServiceParams",
    "Participant",
    "Payload",
    "PayloadKindDto",
    "Relationship",
    "RelationshipKindDto",
    "SdkServiceParams",
    "Service",
    "Session",
    "SessionModeDto",
    "SessionStatusDto",
    "Tag",
    "Term",
    "TermTagsUpdateParams",
    "Tool",
    "ToolId",
    "ToolParameter",
    "ToolParameterDtoEnumItem",
    "ToolParameterTypeDto",
    "ToolServiceKindDto",
    "ValidationError",
    "ValidationErrorLocItem",
]



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Term(UniversalBaseModel):
    """
    Represents a glossary term associated with an agent.

    Use this model for representing complete term information in API responses.
    """

    id: str = pydantic.Field()
    """
    Unique identifier for the term
    """

    name: str = pydantic.Field()
    """
    The name of the term, e.g., 'Gas' in blockchain.
    """

    description: str = pydantic.Field()
    """
    A detailed description of the term
    """

    synonyms: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    A list of synonyms for the term, including alternate contexts if applicable.
    """

    tags: typing.List[str] = pydantic.Field()
    """
    List of tag IDs associated with the term
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.capability import Capability
from ..core.pydantic_utilities import parse_obj_as
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..core.jsonable_encoder import jsonable_encoder
from ..errors.not_found_error import NotFoundError
from ..types.capability_tag_update_params import CapabilityTagUpdateParams
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CapabilitiesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        tag_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[Capability]:
        """
        Retrieves a list of all capabilities in the system.

        Returns an empty list if no capabilities exist.
        Capabilities are returned in no guaranteed order.

        Parameters
        ----------
        tag_id : typing.Optional[str]
            The tag ID to filter capabilities by

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Capability]
            List of all capabilities in the system

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.capabilities.list()
        """
        _response = self._client_wrapper.httpx_client.request(
            "capabilities",
            method="GET",
            params={
                "tag_id": tag_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[Capability],
                    parse_obj_as(
                        type_=typing.List[Capability],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        title: str,
        description: str,
        signals: typing.Sequence[str],
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Capability:
        """
        Creates a new capability in the system.

        The capability will be initialized with the provided title, description, signals, and optional tags.
        A unique identifier will be automatically generated.

        Default behaviors:
        - `signals` defaults to an empty list if not provided

        Parameters
        ----------
        title : str
            The title of the capability

        description : str
            Detailed description of the capability's purpose

        signals : typing.Sequence[str]
            Example signals that this capability can handle

        tags : typing.Optional[typing.Sequence[str]]
            List of tag IDs associated with the capability

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Capability
            Capability successfully created. Returns the complete capability object including generated ID.

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.capabilities.create(
            title="Provide Replacement Phone",
            description="Provide a replacement phone when a customer needs repair for their phone.",
            signals=[
                "My phone is broken",
                "I need a replacement while my phone is being repaired",
            ],
            tags=["tag1", "tag2"],
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "capabilities",
            method="POST",
            json={
                "title": title,
                "description": description,
                "signals": signals,
                "tags": tags,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Capability,
                    parse_obj_as(
                        type_=Capability,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def retrieve(
        self,
        capability_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Capability:
        """
        Retrieves details of a specific capability by ID.

        Returns the complete capability object.

        Parameters
        ----------
        capability_id : str
            Unique identifier for the capability

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Capability
            Capability details successfully retrieved. Returns the complete capability object.

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.capabilities.retrieve(
            capability_id="cap_123abc",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"capabilities/{jsonable_encoder(capability_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Capability,
                    parse_obj_as(
                        type_=Capability,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self,
        capability_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Deletes a capability from the system.

        Deleting a non-existent capability will return 404.
        No content will be returned from a successful deletion.

        Parameters
        ----------
        capability_id : str
            Unique identifier for the capability

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.capabilities.delete(
            capability_id="cap_123abc",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"capabilities/{jsonable_encoder(capability_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        capability_id: str,
        *,
        title: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        signals: typing.Optional[typing.Sequence[str]] = OMIT,
        tags: typing.Optional[CapabilityTagUpdateParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Capability:
        """
        Updates an existing capability's attributes.

        Only the provided attributes will be updated; others will remain unchanged.
        The capability's ID and creation timestamp cannot be modified.

        Parameters
        ----------
        capability_id : str
            Unique identifier for the capability

        title : typing.Optional[str]
            The title of the capability

        description : typing.Optional[str]
            Detailed description of the capability's purpose

        signals : typing.Optional[typing.Sequence[str]]
            Example signals that this capability can handle

        tags : typing.Optional[CapabilityTagUpdateParams]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Capability
            Capability successfully updated. Returns the updated capability.

        Examples
        --------
        from parlant.client import ParlantClient

        client = ParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )
        client.capabilities.update(
            capability_id="cap_123abc",
            title="Provide Replacement Phone",
            description="Provide a replacement phone when a customer needs repair for their phone.",
            signals=[
                "My phone is broken",
                "I need a replacement while my phone is being repaired",
            ],
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"capabilities/{jsonable_encoder(capability_id)}",
            method="PATCH",
            json={
                "title": title,
                "description": description,
                "signals": signals,
                "tags": convert_and_respect_annotation_metadata(
                    object_=tags,
                    annotation=CapabilityTagUpdateParams,
                    direction="write",
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Capability,
                    parse_obj_as(
                        type_=Capability,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncCapabilitiesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        tag_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[Capability]:
        """
        Retrieves a list of all capabilities in the system.

        Returns an empty list if no capabilities exist.
        Capabilities are returned in no guaranteed order.

        Parameters
        ----------
        tag_id : typing.Optional[str]
            The tag ID to filter capabilities by

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Capability]
            List of all capabilities in the system

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.capabilities.list()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "capabilities",
            method="GET",
            params={
                "tag_id": tag_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[Capability],
                    parse_obj_as(
                        type_=typing.List[Capability],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        title: str,
        description: str,
        signals: typing.Sequence[str],
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Capability:
        """
        Creates a new capability in the system.

        The capability will be initialized with the provided title, description, signals, and optional tags.
        A unique identifier will be automatically generated.

        Default behaviors:
        - `signals` defaults to an empty list if not provided

        Parameters
        ----------
        title : str
            The title of the capability

        description : str
            Detailed description of the capability's purpose

        signals : typing.Sequence[str]
            Example signals that this capability can handle

        tags : typing.Optional[typing.Sequence[str]]
            List of tag IDs associated with the capability

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Capability
            Capability successfully created. Returns the complete capability object including generated ID.

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.capabilities.create(
                title="Provide Replacement Phone",
                description="Provide a replacement phone when a customer needs repair for their phone.",
                signals=[
                    "My phone is broken",
                    "I need a replacement while my phone is being repaired",
                ],
                tags=["tag1", "tag2"],
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "capabilities",
            method="POST",
            json={
                "title": title,
                "description": description,
                "signals": signals,
                "tags": tags,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Capability,
                    parse_obj_as(
                        type_=Capability,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def retrieve(
        self,
        capability_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Capability:
        """
        Retrieves details of a specific capability by ID.

        Returns the complete capability object.

        Parameters
        ----------
        capability_id : str
            Unique identifier for the capability

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Capability
            Capability details successfully retrieved. Returns the complete capability object.

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.capabilities.retrieve(
                capability_id="cap_123abc",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"capabilities/{jsonable_encoder(capability_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Capability,
                    parse_obj_as(
                        type_=Capability,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self,
        capability_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Deletes a capability from the system.

        Deleting a non-existent capability will return 404.
        No content will be returned from a successful deletion.

        Parameters
        ----------
        capability_id : str
            Unique identifier for the capability

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.capabilities.delete(
                capability_id="cap_123abc",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"capabilities/{jsonable_encoder(capability_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        capability_id: str,
        *,
        title: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        signals: typing.Optional[typing.Sequence[str]] = OMIT,
        tags: typing.Optional[CapabilityTagUpdateParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Capability:
        """
        Updates an existing capability's attributes.

        Only the provided attributes will be updated; others will remain unchanged.
        The capability's ID and creation timestamp cannot be modified.

        Parameters
        ----------
        capability_id : str
            Unique identifier for the capability

        title : typing.Optional[str]
            The title of the capability

        description : typing.Optional[str]
            Detailed description of the capability's purpose

        signals : typing.Optional[typing.Sequence[str]]
            Example signals that this capability can handle

        tags : typing.Optional[CapabilityTagUpdateParams]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Capability
            Capability successfully updated. Returns the updated capability.

        Examples
        --------
        import asyncio

        from parlant.client import AsyncParlantClient

        client = AsyncParlantClient(
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.capabilities.update(
                capability_id="cap_123abc",
                title="Provide Replacement Phone",
                description="Provide a replacement phone when a customer needs repair for their phone.",
                signals=[
                    "My phone is broken",
                    "I need a replacement while my phone is being repaired",
                ],
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"capabilities/{jsonable_encoder(capability_id)}",
            method="PATCH",
            json={
                "title": title,
                "description": description,
                "signals": signals,
                "tags": convert_and_respect_annotation_metadata(
                    object_=tags,
                    annotation=CapabilityTagUpdateParams,
                    direction="write",
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Capability,
                    parse_obj_as(
                        type_=Capability,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from .gateway_timeout_error import GatewayTimeoutError
from .not_found_error import NotFoundError
from .service_unavailable_error import ServiceUnavailableError
from .unprocessable_entity_error import UnprocessableEntityError

__all__ = [
    "GatewayTimeoutError",
    "NotFoundError",
    "ServiceUnavailableError",
    "UnprocessableEntityError",
]



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
from .legacy_guideline_connection_update_params import (
    LegacyGuidelineConnectionUpdateParams,
)
from .guideline_tool_association_update_params import (
    GuidelineToolAssociationUpdateParams,
)
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class LegacyGuidelineUpdateParams(UniversalBaseModel):
    """
    Parameters for updating Guideline objects.
    """

    connections: typing.Optional[LegacyGuidelineConnectionUpdateParams] = None
    tool_associations: typing.Optional[GuidelineToolAssociationUpdateParams] = None
    enabled: typing.Optional[bool] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
